local net = net
local snet = slib.Components.Network
local table = table
local coroutine = coroutine
local util = util
local istable = istable
local isstring = isstring
local string = string
local ipairs = ipairs
local tostring = tostring
local RealTime = RealTime
local hook = hook
local notification = notification
--
slib.Storage.Network.bigdata = slib.Storage.Network.bigdata or {}

local function getNetParts(text, max_size)
	local parts = {}

	for i = 1, #text, max_size do
		parts[#parts + 1] = text:sub(i, i + max_size - 1)
	end

	local net_parts = {}

	for _, string_part in ipairs(parts) do
		local compressed_data = util.Compress(string_part)
		local compressed_length = string.len(compressed_data)

		table.insert(net_parts, {
			data = compressed_data,
			length = compressed_length
		})

		coroutine.yield()
	end

	return coroutine.yield(net_parts)
end

local uid = 0

snet.InvokeBigData = function(request, ply, data, max_size, progress_text, progress_id)
	local request_data = ''
	local backward = request.backward
	local id = request.id
	local name = request.name

	if istable(data) then
		request_data = util.TableToJSON({
			backward = backward,
			id = id,
			data_type = 'table',
			data = util.TableToJSON(snet.GetNormalizeDataTable(data))
		})
	elseif isstring(data) then
		request_data = util.TableToJSON({
			backward = backward,
			id = id,
			data_type = 'string',
			data = data
		})
	else
		return
	end

	if CLIENT then
		for _, v in ipairs(slib.Storage.Network.bigdata) do
			if v.name == name then return end
		end
	else
		for _, v in ipairs(slib.Storage.Network.bigdata) do
			if v.name == name and v.ply == ply then return end
		end
	end

	progress_text = progress_text or ''
	progress_id = progress_id or tostring(util.CRC(RealTime() .. 'bigdata' .. progress_text))
	max_size = max_size or 10000
	uid = uid + 1
	local hook_name = 'Slib_NetBigDataSender_' .. name .. uid
	local thread = coroutine.create(getNetParts)

	local index = table.insert(slib.Storage.Network.bigdata, {
		id = id,
		name = name,
		ply = ply,
		net_parts = nil,
		max_parts = nil,
		current_part = 0,
		progress_id = progress_id,
		progress_text = progress_text,
	})

	if SERVER then
		hook.Run('SlibPreparingBigdataSending', ply, name)
	else
		if progress_id ~= '' and progress_text ~= '' then
			notification.AddProgress('SlibBigDataPreparing_' .. name, 'Data is being prepared for upload...')
		end

		hook.Run('SlibPreparingBigdataSending', LocalPlayer(), name)
	end

	hook.Add('Think', hook_name, function()
		if coroutine.status(thread) == 'dead' then
			table.remove(slib.Storage.Network.bigdata, index)
			hook.Remove('Think', hook_name)

			if SERVER then
				hook.Run('SlibStopBigdataSending', ply, name)
			else
				if progress_id ~= '' and progress_text ~= '' then
					notification.Kill('SlibBigDataPreparing_' .. name)
					notification.AddLegacy('Failed to pack data to send!', NOTIFY_ERROR, 4)
				end

				hook.Run('SlibStopBigdataSending', LocalPlayer(), name)
			end

			return
		end

		local worked, result = coroutine.resume(thread, request_data, max_size)
		if result == nil then return end
		hook.Remove('Think', hook_name)
		local net_parts = result
		local max_parts = #net_parts
		if net_parts == nil or #net_parts == 0 then return end
		slib.Storage.Network.bigdata[index].net_parts = net_parts
		slib.Storage.Network.bigdata[index].max_parts = max_parts

		request.Eternal()
		snet.AddRequestToList(request)

		if SERVER then
			net.Start('slib_cl_bigdata_receive')
			net.WriteString(name)
			net.WriteInt(index, 10)
			net.WriteInt(max_parts, 10)
			net.WriteString(progress_id)
			net.WriteString(progress_text)
			net.Send(ply)
		else
			net.Start('slib_sv_bigdata_receive')
			net.WriteString(name)
			net.WriteInt(index, 10)
			net.WriteInt(max_parts, 10)
			net.SendToServer()

			if progress_id ~= '' and progress_text ~= '' then
				notification.Kill('SlibBigDataPreparing_' .. name)
			end
		end

		hook.Run('SnetBigDataStartSending', ply, name)
	end)
end